(define-module (ordo action quadlet)
  #:use-module (ice-9 filesystem)
  #:use-module (ini)
  #:use-module (logging logger)
  #:use-module (ordo connection)
  #:use-module ((ordo action filesystem) #:prefix fs:)
  #:use-module ((srfi srfi-1) #:select (remove))
  #:export (create-network
            create-pod
            create-container
            create-volume
            create-image
            create-build))

(define quadlet-dir "/etc/containers/systemd")

(define default-install-options '(("WantedBy" . "multi-user.target default.target")))

(define (scm->ini-string data)
  (with-output-to-string (lambda () (scm->ini data))))

(define (build-quadlet quadlet-type name description unit-options quadlet-options service-options install-options)
  (let* ((description (or description (string-append "Podman " (string-downcase quadlet-type) " " name)))
         (data  `(("Unit" ("Description" . ,description) ,@unit-options)
                  (,(string-titlecase quadlet-type) ,@quadlet-options)
                  ,@(if (null? service-options) '() (list (cons "Service" service-options)))
                  ,@(if (null? install-options) '() (list (cons "Install" install-options))))))
    (scm->ini-string data)))

(define-syntax define-quadlet-type
  (syntax-rules ()
    ((define-quadlet-type function-name quadlet-type suffix default-install-options)
     (define* (function-name conn
                             #:key name description
                             (quadlet-options '())
                             (unit-options '())
                             (service-options '())
                             (install-options default-install-options))
       (fs:install-file conn
                        #:path (file-name-join* quadlet-dir (string-append name suffix))
                        #:content (build-quadlet quadlet-type name description quadlet-options unit-options service-options install-options))))))

(define-quadlet-type create-network "Network" ".network" default-install-options)

(define-quadlet-type create-pod "Pod" ".pod" default-install-options)

(define-quadlet-type create-container "Container" ".container" default-install-options)

(define-quadlet-type create-volume "Volume" ".volume" '())

(define-quadlet-type create-build "Build" ".build" '())

(define-quadlet-type create-image "Image" ".image" '())
